##############################################################################
#   
#  Setup script for the ESO Scisoft scientific software collection
#
#  This file should be sourced using the sh/ksh/bash dot "." command
#
#      source /scisoft/bin/Setup.csh
#
#  by all users of the software. They should be no need to do anything else.
#   
#  If you encounter any problems, email scihelp@eso.org.
#   
##############################################################################
#   
#  Revision history
#   
#  ??/11/2000 - kmaguire - created
#  07/06/2005 - ahuxley  - converted to .src with substitutable markers, 
#                          corrected eclipse and python root directory tests,
#  
##############################################################################

#
#  echo'ing in scp sessions not allowed, so use this function instead of
#  echo.
#

alias echo_if_tty '[ -t 1 ] && echo "\!:1"'

#
#  Before doing anything check the shell
#

if ( $?shell && `basename $shell` != tcsh && `basename $shell` != csh ) then
    echo_if_tty "please start a csh/tcsh shell before setting up scisoft"
    exit -1
endif

#
#  Find out which arch we are running on
#

if ( `uname` != Linux ) then
    echo_if_tty "unsupported architecture"
    exit -1
endif

#
#  Add the main Scisoft binaries to the command search path
#

if ( $?PATH ) then
    setenv PATH /scisoft/bin:${PATH}
else
    setenv PATH /scisoft/bin
endif

#
#  Add the main Scisoft libraries to the library search path
#

if ( $?LD_LIBRARY_PATH ) then
    setenv LD_LIBRARY_PATH /scisoft/lib:${LD_LIBRARY_PATH}
else
    setenv LD_LIBRARY_PATH /scisoft/lib
endif

#
#  Add the main Scisoft manual pages to the manual page search path
#

if ( $?MANPATH ) then
    setenv MANPATH /scisoft/man:${MANPATH}
else
    setenv MANPATH /scisoft/man:
endif

#
#  Add the Scisoft X application defaults directory to XUSERFILESEARCHPATH.
#

if ( $?XUSERFILESEARCHPATH ) then
    setenv XUSERFILESEARCHPATH /scisoft/lib/X11/app-defaults:${XUSERFILESEARCHPATH}
else
    setenv XUSERFILESEARCHPATH /scisoft/lib/X11/app-defaults
endif

#
#  IRAF related definitions
#

if ( -d /scisoft/share/iraf ) then
    #  Note trailing slashes
    setenv iraf /scisoft/share/iraf/iraf/
    setenv stsdas /scisoft/share/iraf/extern/stsdas/
    setenv tables /scisoft/share/iraf/extern/tables/
    if ( -f ${iraf}unix/hlib/irafuser.csh ) then
	source ${iraf}unix/hlib/irafuser.csh
        #  The above script sets F77, which is quite naughty of it; 
        #  it can cause users' own compiles of totally unrelated software
        #  to fail.
        setenv F77_IRAF $F77
	unsetenv F77
    endif
    setenv IRAFARCH redhat

    #  From: "Warren J. Hack" <hack@stsci.edu>
    #  To: Richard Hook <rhook@eso.org>
    #  CC: greenfield@stsci.edu
    #  Subject: Re: Greetings and a question
    #  
    #  Hello,
    #
    #  >I just had a funny problem on my new machine here and I thought I
    #  >would check if you had seen it before and had a simple answer. I have
    #  >a new Linux FC3 machine which has IRAF 2.12.2a (redhat). I also have
    #  >PyRAF 1.1.2 (2005Mar19)  and Python 2.4. The OS is
    #  >Fedora Core release 3 (Heidelberg). If I start the cl all seems well,
    #  >however, from Pyraf I get:
    #  >
    #  >--> imstat dev$pix
    #  >#               IMAGE      NPIX      MEAN    STDDEV       MIN       MAX
    #  >             dev$pix    262144     108.3     131.3       -1.    19936.
    #  >
    #  >PANIC in `/scisoft/share/iraf/iraf/bin.redhat/x_images.e': Memory has
    #  >been corrupted
    #  >Killing IRAF task `imstatistics'
    #  >Traceback (innermost last):
    #  > File "<CL script CL1>", line 1, in ?
    #  >   iraf.imstat('dev$pix', _save=1)
    #  >IrafError: Error running IRAF task imstatistics
    #  >('Not a legal IRAF pipe record', 32, 'Broken pipe')
    #  >--> .exit
    #  >
    #  >which is very strange, as this is exactly the same IRAF (as far as I
    #  >can see) to the one I get from the CL.
    #  >
    #  >I half suspect this is an environment rather than fundamental problem,
    #  >but don't know where to look.
    #  >
    #  >Any ideas?
    #  >
    #  >Cheers, Richard
    #
    #  The error you report appears similar to one that Nadia recognized for
    #  another help call.  It is a known IRAF problem on some platforms with
    #  the solution being to add the following line to your ~/.login file (or
    #  whatever unix login file serves the same purpose):
    #  
    #  limit stacksize unlimited
    #  
    #  This should correct the problem.  In the meantime, I hope it cools off
    #  for you soon, or at the very least, that the humidity moves out.
    #  
    #  Cheers,
    #  Warren
    limit stacksize unlimited

endif

#
#  IDL related definitions
#

if ( -d /scisoft/share/idl ) then
    #  Marco Lombardi sorted out Harald Kuntschner's problem with
    #  strsplit() which was because libraries in /scisoft/share/idl/idl/lib
    #  are not included in IDL_PATH. Actually, rather than including a long
    #  list of specific subdirectories of this, then we just include the whole
    #  tree.

    foreach IDLPKG(astron jhuapl starfinder atv)
        if ( -d /scisoft/share/idl/$IDLPKG ) then
                if ( $?IDL_PATH ) then
                        if ( -z "$IDL_PATH" )  then
                                setenv IDL_PATH "<IDL_DEFAULT>"
                        endif
                else
                        setenv IDL_PATH "<IDL_DEFAULT>"
                endif

                setenv IDL_PATH ${IDL_PATH}:+/scisoft/share/idl/${IDLPKG}
        endif
    end

    foreach IDLDIR(/scisoft/share/idl/idl)
        if ( -d $IDLDIR ) then
            setenv IDL_DIR ${IDLDIR}
        endif
    end

    #  Marco wants some settings related to the display for ESO only.
    if ( -f /scisoft/share/idl/startup-eso.idl ) then
        setenv IDL_STARTUP /scisoft/share/idl/startup-eso.idl
    endif
endif

#
#  Eclipse
#

if ( -d /scisoft/share/eclipse ) then
    setenv E_PATH /scisoft/share/eclipse/eclipse
    setenv E_VERBOSE 1
    setenv E_MAXMEM  64
    setenv E_MAXSWAP 128
    setenv E_TMPDIR  .
    unsetenv E_DEBUG
    if ( $?PATH ) then
        setenv PATH ${PATH}:${E_PATH}/bin
    else
        setenv PATH ${E_PATH}/bin
    endif
    if ( $?MANPATH ) then
	setenv MANPATH ${MANPATH}:${E_PATH}/man
    else
	setenv MANPATH ${E_PATH}/man:
    endif
endif

#
#  Tiny Tim HST psf software
#

if ( -d /scisoft/share/tinytim ) then
    setenv TINYTIM /scisoft/share/tinytim
endif

#
#  PGPLOT graphics software
#

setenv PGPLOT_DIR /scisoft/share/pgplot
setenv PGPLOT_FONT /scisoft/share/pgplot/grfont.dat
setenv PGPLOT_DEV /xwin

#
#  MIDAS
#

if ( -d /scisoft/lib/midas ) then
    setenv MIDASHOME /scisoft/lib/midas
    setenv MIDVERS 08SEPpl1.0
endif

#
#  XPA - disable warning at startup
#

setenv XPA_VERSIONCHECK 0

#
#  Ggobi
#

setenv GGOBI_HOME /scisoft/lib/ggobi

#
#  Setup a search path for Latex to include astronomical journals
#

if ( $?TEXINPUTS ) then
    setenv TEXINPUTS ${TEXINPUTS}:`echo /scisoft/share/texmf/* | sed 's/ /:/g'`
else
    #  Note the leading ':' which means that latex should search the
    #  default paths.
    setenv TEXINPUTS :`echo /scisoft/share/texmf/* | sed 's/ /:/g'`
endif

#
#  Python
#

if ( -d /scisoft/share/python ) then
    if ( $?PYTHONPATH ) then
        setenv PYTHONPATH ~/PythonModules:${PYTHONPATH}
    else
        setenv PYTHONPATH ~/PythonModules
    endif
endif

#
#  DISLIN stuff
#

setenv DISLIN /scisoft/share/dislin

#
#  Gildas
#

setenv GAG_PATH /scisoft/share/pc-fedora6-g95/etc

#
#  Check whether there is a /scilocal available and if so execute
#  its Setup.csh file
#

if ( -f /scilocal/bin/Setup.csh ) then
    source /scilocal/bin/Setup.csh
endif

#
# Miriad
#

if ( -f /scisoft/share/miriad/miriad_start.csh ) then
     # prevent Miriad form displaying it's "hello" message
    setenv MIRHELLO x
    source /scisoft/share/miriad/miriad_start.csh
endif

#
# Karma
#

if ( -f /scisoft/share/karma/.login ) then
    source /scisoft/share/karma/.login
endif
setenv LD_LIBRARY_PATH ${LD_LIBRARY_PATH}:/scisoft/share/karma/lib/

#
# Gaia
#

setenv GAIA_DIR /scisoft/share/star/bin/gaia
setenv ESP_DIR /scisoft/share/star/bin/esp
setenv EXTRACTOR_DIR /scisoft/share/star/bin/extractor
setenv CONVERT_DIR /scisoft/share/star/bin/convert
setenv KAPPA_DIR /scisoft/share/star/bin/kappa
setenv PHOTOM_DIR /scisoft/share/star/bin/photom
setenv POLPACK_DIR /scisoft/share/star/bin/polpack

# 
# Check if SELinux is enabled
# 

if ( ! $?SCISOFT_IGNORE_SELINUX_WARNING ) then
    if ( -e /usr/sbin/sestatus ) then
        if ( `/usr/sbin/sestatus | grep enabled` != "" ) then

            echo "Warning: SELinux is currently enabled on your computer."
            echo "This will prevent some of Scisoft's software such as PyRAF from running."
            echo "More information is avaiable in the Scisoft FAQ:"
            echo "http://www.eso.org/sci/data-processing/software/scisoft7/faq.html#q25"
            echo ""
            echo "This warning can be disabled by setting the environment variable SCISOFT_IGNORE_SELINUX_WARNING to '1'"
        endif
    endif
endif


